<?php
/**
* @package Fifny-cloud-configurators
*/
namespace FCC_Inc\FCC_Base;

use FCC_Inc\FCC_Base\FCC_BaseInformation;

class FCC_Endpoint extends FCC_BaseInformation
{
    private $configurationSymbol;
    private $configurationQuantity;

    public function __construct(){

        register_rest_route('fifny-cloud-configurators/v1', 'add/product/(?P<configurationSymbol>[0-9]+)/(?P<configurationQuantity>[0-9]+)', array(
           'methods'                => 'GET',
           'callback'               => array($this, 'fifny_create_product'),
           'args' => array(
                'configurationSymbol' => array(
                    'validate_callback' => array($this, 'is_numeric')
                ),
                'configurationQuantity' => array(
                    'validate_callback' => array($this, 'is_numeric')
                ),
            ),
           'permission_callback'    => array($this, 'permission_check_is_configuration_exsist')
        )
      );
    }

    public function fifny_create_product(){
        
        //Get configration details from cloud
        $combinationDetials = $this->get_combination_details();
        
        //Prepare parametrs to insert post 
        $product_name = 'Konfigurator Fifny Cloud - ' . $combinationDetials->item . ' - ' . $this->get_combination_symbol();
        
        //Get the path of saved preview image and prepare URL 
        $image_url = ''; 
        foreach($combinationDetials->slots as $slot){
            //print_r($slot); 
            if($slot->symbol === 'Preview'){
                if(!isset($slot->propertiesToSave)){
                    return new \WP_Error( 'faild-get-image', esc_html__( 'No preview slot found.', 'fifny-cloud-configurators'), array( 'status' => 500 ) ); 
                }
                foreach($slot->propertiesToSave as $property){
                    if($property->symbol === 'componentImagePreview'){
                        $image_url = $this->get_cloud_shop_url() . '/' . $property->value;
                    }
                } 
            }
        }
        
        if(empty($image_url)){
             return new \WP_Error( 'faild-get-image', esc_html__( 'Faild to get image preview. Image path empty.', 'fifny-cloud-configurators'), array( 'status' => 500 ) ); 
        }
        
        $data = [
            'post_title' => $product_name,
            'post_status' => 'publish',
            'post_type' => 'product',
            'meta_input' => array(
                'fifny_cc_symbol' => $this->get_combination_symbol(),
                'fifny_cc_item'   => $combinationDetials->item, 
            )
        ];

        /*
        ** Insert post with $data args
        */
        $post_id = wp_insert_post( $data );
       
        if( is_wp_error( $post_id ) ){
            return new \WP_Error( 'faild-create', esc_html__( $post_id->get_error_message(), 'fifny-cloud-configurators'), array( 'status' => 500 ) ); 
        }
       
        //Get $product object from post ID
        $product = wc_get_product( $post_id );
         
        //Set others product details
        $product->set_catalog_visibility('hidden');
        $product->set_price( $combinationDetials->sum );
        $product->set_regular_price( $combinationDetials->sum );   
       
        //Save product details
        $product->save();

        /*
        ** Set product image 
        */
        
        //Required include it to use media_sideload_image outside the /wp-admin
        require_once(ABSPATH . 'wp-admin/includes/media.php');
        require_once(ABSPATH . 'wp-admin/includes/file.php');
        require_once(ABSPATH . 'wp-admin/includes/image.php');
        
        set_post_thumbnail($post_id, media_sideload_image($image_url, $post_id, $product_name, $return = 'id' )); 

        /*
        ** Redirect user to product add to basket URL
        ** URL of cart can be different so check it
        */
        if(function_exists('wc_get_cart_url'))
            $cart_url = wc_get_cart_url();
        else
            $cart_url = '/cart/';

        $url = $cart_url . "?add-to-cart=${post_id}&quantity=" . $this->get_combination_quantity();

        if( wp_safe_redirect($url)) {
            exit;
        }
    }
    
    public function permission_check_is_configuration_exsist($request){
        
        /*
        ** Check is configuration with that symbol exists in cloud
        */
    
        $this->configurationSymbol = $this->get_combination_symbol_by_request($request);
        $this->configurationQuantity = $this->get_combination_quantity_by_request($request);

        //Get the details by symbol
        $body = $this->get_combination_details(); 
        
        //Check if symbol exsist - if not return false
        if(isset($body->containsErrors) && $body->containsErrors == true){
            return new \WP_Error( 'combination-not-found', esc_html__( 'Combination not found in remote repository.', 'fifny-cloud-configurators'), array( 'status' => 404 ) );
        }
        
        //Check if product with this symbol not exists in wordpress 
        if(!($this->check_if_product_not_exsist_in_local_shop())){
            return new \WP_Error( 'combination-already-exsist', esc_html__( 'Combination already exsist in local shop.', 'fifny-cloud-configurators'), array( 'status' => 400 ) );
        } 
        return true; 
    }
    
    public function check_if_product_not_exsist_in_local_shop(){
        
        $query_object = $this->get_post_by_symbol(); 
        
        if($query_object->post_count === 0){
            return true; 
        }
        return false; 
    }
    
    public function get_post_by_symbol(){
        
        $args = array(
            'post_type'      => 'product',
            'meta_query' => array(
                array(
                    'key'       => 'fifny_cc_symbol', 
                    'value'     => $this->get_combination_symbol(),
                    'compare'   => '='
                )
            )
        );

        return new \WP_Query( $args );
    }
    
    public function get_combination_details(){
        /*
        ** Get configuration detils from cloud
        */
        $configurationSymbol = $this->get_combination_symbol();
        
        $request = wp_remote_get( $this->get_cloud_shop_url() . "/do/jsonCombination?symbol=${configurationSymbol}" ); 
        
        if( is_wp_error( $request ) ){
            return false; 
        }
                                 
        $body = wp_remote_retrieve_body( $request ); 
        
        $data = json_decode( $body ); 
        
        return $data; 
    }
    
    public function get_combination_symbol(){
        return $this->configurationSymbol; 
    }

    public function  get_combination_quantity(){
        return $this->configurationQuantity;
    }
    
    public function get_combination_symbol_by_request($request){
         return $request->get_param('configurationSymbol'); 
    }

    public function get_combination_quantity_by_request($request){
        return $request->get_param('configurationQuantity');
    }
    
    public function is_numeric( $param ){
        return is_numeric( $param );
    }
}
